#!/usr/bin/env python
# -*- coding: utf-8 -*-

import wiringpi as wpi

import time
import Adafruit_PCA9685

pwm = Adafruit_PCA9685.PCA9685()
servo_min = 150  # Min pulse length out of 4096
servo_max = 600  # Max pulse length out of 4096
pwm.set_pwm_freq(60)
deg_max = 150 # Max deg
deg_min = 30 # Min deg

wpi.wiringPiSetupGpio()

R_IN1 = 17
R_IN2 = 27
L_IN1 = 22
L_IN2 = 10
BZ = 11
SOL = 9
BAT_LOW = 4
PAYLOAD_IN1 = 0
PAYLOAD_IN2 = 5
PAYLOAD_IN3 = 6
PAYLOAD_OUT1 = 16
PAYLOAD_OUT2 = 20
PAYLOAD_OUT3 = 21

INPUT = 0
OUTPUT = 1
LOW = 0
HIGH = 1
PUD_OFF = 0
PUD_DOWN = 1
PUD_UP = 2
STOP = 0
FW = 1
RW = 2
CW = 3
CCW = 4
BRK = 9
OFF = 0
ON = 1

wpi.softToneCreate(BZ)
wpi.softPwmCreate(R_IN1,0,100)
wpi.softPwmCreate(R_IN2,0,100)
wpi.softPwmCreate(L_IN1,0,100)
wpi.softPwmCreate(L_IN2,0,100)
wpi.pinMode(SOL,OUTPUT)
wpi.pinMode(BAT_LOW,INPUT)
wpi.pullUpDnControl(BAT_LOW,PUD_UP)

wpi.pinMode(PAYLOAD_IN1,INPUT)
wpi.pinMode(PAYLOAD_IN2,INPUT)
wpi.pinMode(PAYLOAD_IN3,INPUT)
wpi.pinMode(PAYLOAD_OUT1,OUTPUT)
wpi.pinMode(PAYLOAD_OUT2,OUTPUT)
wpi.pinMode(PAYLOAD_OUT3,OUTPUT)
wpi.digitalWrite(PAYLOAD_OUT1,LOW)
wpi.digitalWrite(PAYLOAD_OUT2,LOW)
wpi.digitalWrite(PAYLOAD_OUT3,LOW)

def deg2pls(deg):
    data = int((deg-deg_min) *  (servo_max-servo_min) / (deg_max - deg_min)) + servo_min
    return data

def setServo(ch,deg):
    if ch <16:
        pwm.set_pwm(ch, 0,deg2pls(deg))
    else:
        print("Channel Select Error")

def beep(time,n):
    for i in range(n):
        wpi.softToneWrite(BZ,1500)
        wpi.delay(time/2)
        wpi.softToneWrite(BZ,0)
        wpi.delay(time/2)

def Pl_port_in(ch):
    if ch == 1:
        return wpi.digitalRead(PAYLOAD_IN1)
    elif ch == 2:
        return wpi.digitalRead(PAYLOAD_IN2)
    elif ch == 3:
        return wpi.digitalRead(PAYLOAD_IN3)
    else:
        pass

def Pl_port_out(ch,com):
    if ch == 1:
        wpi.digitalWrite(PAYLOAD_OUT1,com)
    elif ch == 2:
        wpi.digitalWrite(PAYLOAD_OUT2,com)
    elif ch == 3:
        wpi.digitalWrite(PAYLOAD_OUT3,com)
    else:
        pass


def Bat_check():
    if wpi.digitalRead(BAT_LOW) == HIGH:
        return False
    else:
        return True

def Move_Cntrl(com,val):
    beep(20,2)
    print("Battery Check")
    if Bat_check():
        beep(100,5)
        print("OK")
    else:
        beep(5000,2)
        print("Low Battery")
    beep(1000,2)
    wpi.delay(1000)

    if com == FW:
        wpi.softPwmWrite(R_IN1,val)
        wpi.softPwmWrite(R_IN2,0)
        wpi.softPwmWrite(L_IN1,val)
        wpi.softPwmWrite(L_IN2,0)
    elif com == RW:
        wpi.softPwmWrite(R_IN1,0)
        wpi.softPwmWrite(R_IN2,val)
        wpi.softPwmWrite(L_IN1,0)
        wpi.softPwmWrite(L_IN2,val)
    elif com == CW:
        wpi.softPwmWrite(R_IN1,val)
        wpi.softPwmWrite(R_IN2,0)
        wpi.softPwmWrite(L_IN1,0)
        wpi.softPwmWrite(L_IN2,val)
    elif com == CCW:
        wpi.softPwmWrite(R_IN1,0)
        wpi.softPwmWrite(R_IN2,val)
        wpi.softPwmWrite(L_IN1,val)
        wpi.softPwmWrite(L_IN2,0)
    elif com == BRK:
        wpi.delay(10)
        wpi.softPwmWrite(R_IN1,100)
        wpi.softPwmWrite(R_IN2,100)
        wpi.softPwmWrite(L_IN1,100)
        wpi.softPwmWrite(L_IN2,100)
    else:
        wpi.softPwmWrite(R_IN1,0)
        wpi.softPwmWrite(R_IN2,0)
        wpi.softPwmWrite(L_IN1,0)
        wpi.softPwmWrite(L_IN2,0)

def Sol_Cntrl(com):
    if com == ON:
        wpi.digitalWrite(SOL,HIGH)
    else:
        wpi.digitalWrite(SOL,LOW)

def main():
    beep(20,2)
    print("Payload Output check")
    Pl_port_out(1,HIGH)
    beep(500,2)
    Pl_port_out(2,HIGH)
    beep(500,2)
    Pl_port_out(3,HIGH)
    beep(500,2)
    wpi.delay(2000)
    Pl_port_out(1,LOW)
    beep(500,2)
    Pl_port_out(2,LOW)
    beep(500,2)
    Pl_port_out(3,LOW)
    beep(1000,1)

    beep(20,2)
    print("Servo Moter Check")
    setServo(0,120)
    wpi.delay(1000)
    setServo(0,60)
    wpi.delay(1000)
    setServo(0,90)
    wpi.delay(1000)
    beep(20,1)
    setServo(1,120)
    wpi.delay(1000)
    setServo(1,60)
    wpi.delay(1000)
    setServo(1,90)
    wpi.delay(1000)
    beep(1000,1)
    wpi.delay(1000)

    print("Solenoid Output check")
    beep(20,2)
    Sol_Cntrl(ON)
    wpi.delay(1000)
    Sol_Cntrl(OFF)
    beep(200,1)

    wpi.delay(1000)

    print("Moter Output check")
    Move_Cntrl(FW,100)
    wpi.delay(2000)
    beep(20,1)

    Move_Cntrl(STOP,0)
    beep(20,1)

    Move_Cntrl(RW,50)
    wpi.delay(2000)
    beep(20,1)

    Move_Cntrl(STOP,0)
    beep(20,1)

    Move_Cntrl(CW,20)
    wpi.delay(2000)
    beep(20,1)

    Move_Cntrl(STOP,0)
    beep(20,1)

    Move_Cntrl(CCW,20)
    wpi.delay(2000)
    beep(20,1)

    Move_Cntrl(STOP,0)
    beep(2000,1)

    print("end")

if __name__ == "__main__":
    main()
